#ifndef _VS1011_H_
#define _VS1011_H_





#define testkk 0

//=====================================
/* ---------- hardwareeinstellungen ----------
 * Verschiedene Hardware-Defines.
 */
#define VS1011_CLEAR_XCS() pinCLR(0,22)
#define VS1011_SET_XCS() pinSET(0,22)
#define VS1011_CLEAR_XDCS() pinCLR(0,20)
#define VS1011_SET_XDCS() pinSET(0,20)

// Frequenz: f_osc / 8
#define VS1011_SET_SPI_CLOCK_READ()	;//	SPCR |= (1 << SPR0); \

// Schreibt-Takt = Lese-Takt
#define VS1011_SET_SPI_CLOCK_WRITE()	;//VS1011_SET_SPI_CLOCK_READ()

/* ----------------------- Makros für Pin-Konfiguration---------------------- */
#define LED_CONFIGURE_PINS()		;//nothing
#define LED_SET_GREEN()				led1_on;led2_off;
#define LED_SET_RED()				led1_off;led2_on;
#define LED_TOGGLE()				led1_tog;led2_tog;
#define LED_DEACTIVATE()			led1_off;led2_off;



/* ------------------------- Debug-Einstellungen --------------------------------
 *
 * Gibt an, welche Bereiche Debugmeldungen ausgeben sollen.
 *
 */

//#define DEBUG					// Debugging-Meldungen für Bootvorgang / Initialisierungen
//#define DEBUG_LCD				// Debugging-Meldungen für LCD-Library
//#define DEBUG_GUI				// Debugging-Meldungen für GUI
//#define DEBUG_SD				// Debugging-Meldungen für SD-Karte
//#define DEBUG_VS1011			// Debugging-Meldungen für VS1011-Ansteuerung
//#define DEBUG_PLAYER			// Debugging-Meldungen für Player
//#define DEBUG_INPUT			// Debugging-Meldungen für Tasteneingabe
//#define DEBUG_FILE_BROWSER	// Debugging-Meldungen für Datei-Browser



/* ------------------------ allgemeine Einstellungen ------------------------ */


// Gibt die Genauigkeit des Timers an: 1000 / GLOBAL_TIMER_DIVISOR = ms
#define GLOBAL_TIMER_DIVISOR 				40 //25 ms

// Maximal unterstützte Dateinamenlänge für einen FAT Eintrag
#define FILENAME_MAXLEN						128

// Größe eines Puffer-Frames, das der Player von SD-Karte läd und an den den VS1011 sendet
#define PLAYER_SEND_BUFFER_SIZE				64

// Größe des Puffers, der zum lesen von ID3 Tags verwendet wird
// Muss mindestens 30 Byte groß sein! Maximale ID3v2 Feld-Lesegröße wird zusätzlich
// in id3.h definiert (ID3_MAXIMAL_ID3V2_FIELD_SIZE)
#define ID3_BUFFER_SIZE						64

// Text, der während des ausschaltens angezeigt wird
#define SHUTDOWN_TEXT						"Goodbye"

// Die folgenden Konstanen geben an, ob der Player die Wiedergabe forsetzt, falls
// der File Browser bzw. das Hauptmenü angezeigt werden. Ansonsten wird die
// Wiedergabe auf Pause gesetzt
#define PLAYER_PAUSE_IN_FILE_BROWSER		true
#define PLAYER_PAUSE_IN_MAIN_MENU			false


/* ------------------- Einstellungen für GUI der Bootsequenz ---------------- */

// Gibt an, ob die Boot-Animation angezeigt werden soll
#define BOOTSEQUENCE_SHOW_ANIMATION			true

// Font für Text in Bootsequenz
#define BOOTSEQUENCE_FONT					gui_elements_pFont

// Y-Koordinate für Meldungstext
#if BOOTSEQUENCE_SHOW_ANIMATION
	#define BOOTSEQUENCE_MESSAGE_TEXT_Y		120
#else
	#define BOOTSEQUENCE_MESSAGE_TEXT_Y		(LCD_BOTTOM / 2 - lcd_getFontHeight(BOOTSEQUENCE_FONT))
#endif

// Meldungstext, der angezeigt wird, wenn keine SD-Karte erkannt wurde
#define BOOTSEQUENCE_NO_SD_CARD_TEXT		"Bitte SD-Karte einlegen"

// Y-Koordinate für Versionsinformations-Text
#define BOOTSEQUENCE_VERSION_TEXT_Y			(LCD_BOTTOM - 20)

#define BOOTSEQUENCE_LOGO_Y					50


/* --------------------------- Werteeinstellungen ------------------------------
 *
 * Gibt an, welche Wertebereiche für verschiedene Einstellungen gültig sind.
 *
 */
#define LCD_BACKLIGHT_MIN 0
#define LCD_BACKLIGHT_DEFAULT 100
#define LCD_BACKLIGHT_MAX 200

#define LCD_BACKLIGHT_TIME_MIN 0
#define LCD_BACKLIGHT_TIME_DEFAULT 10
#define LCD_BACKLIGHT_TIME_MAX 31

#define PLAYER_VOLUME_MIN 0
#define PLAYER_VOLUME_DEFAULT 20
#define PLAYER_VOLUME_MAX 40

#define PLAYER_BASS_AMPLITUDE_MIN 0
#define PLAYER_BASS_AMPLITUDE_DEFAULT 0
#define PLAYER_BASS_AMPLITUDE_MAX 15

#define PLAYER_BASS_FREQUENCY_MIN 0
#define PLAYER_BASS_FREQUENCY_DEFAULT 60
#define PLAYER_BASS_FREQUENCY_MAX 15

// Von den Werten wird noch jeweils 8 subtrahiert!
#define PLAYER_TREBLE_AMPLITUDE_MIN 0
#define PLAYER_TREBLE_AMPLITUDE_DEFAULT 8
#define PLAYER_TREBLE_AMPLITUDE_MAX 15

#define PLAYER_TREBLE_FREQUENCY_MIN 0
#define PLAYER_TREBLE_FREQUENCY_DEFAULT 10
#define PLAYER_TREBLE_FREQUENCY_MAX 15

#define GUI_THEME_MIN 0
#define GUI_THEME_DEFAULT 0
#define GUI_THEME_MAX 2

// Anzahl der Sekunden, nach denen das Player-Menü wieder auf das Wiedergabekontroll
// Menü gesetzt wird
#define PLAYER_MENU_RESET_TIMER 8


// Zeichenkette, die benutzt wird um Fehler in der GUI zu signalisieren (bspw.
// Fehler beim Laden von Menüeinträgen)
#define TEXT_ERROR		"FEHLER"


//=====================================

/* ---------- hardwareeinstellungen ----------
 * Verschiedene Hardware-Defines.
 */
#define POWEROFF_DDR 	DDRD
#define POWEROFF_PORT 	PORTD
#define POWEROFF_PIN	0


/* ----------------------- Makros für Pin-Konfiguration---------------------- */

// SPI
#define SPI_CONFIGURE_PIN_MOSI()	DDRB |= (1 << PB5)
#define SPI_CONFIGURE_PIN_MISO()	DDRB &= ~(1 << PB6)
#define SPI_CONFIGURE_PIN_SCK()		DDRB |= (1 << PB7)
#define SPI_ENABLE()				PORTB |= (1 << PB4); \
									SPCR |= (1 << SPE) | (1 << MSTR)

// LED
#define LED_CONFIGURE_PINS()		DDRC = (1 << PC7) | (1 << PC3)
#define LED_SET_GREEN()				PORTC &= ~(1 << PC7); PORTC |= (1 << PC3)
#define LED_SET_RED()				PORTC &= ~(1 << PC3); PORTC |= (1 << PC7)
#define LED_TOGGLE()				PORTC ^= (1 << PC7) | (1 << PC3)
#define LED_DEACTIVATE()			PORTC &= ~((1 << PC7) | (1 << PC3))



/* ------------------------- Debug-Einstellungen --------------------------------
 *
 * Gibt an, welche Bereiche Debugmeldungen ausgeben sollen.
 *
 */

//#define DEBUG					// Debugging-Meldungen für Bootvorgang / Initialisierungen
//#define DEBUG_LCD				// Debugging-Meldungen für LCD-Library
//#define DEBUG_GUI				// Debugging-Meldungen für GUI
//#define DEBUG_SD				// Debugging-Meldungen für SD-Karte
//#define DEBUG_VS1011			// Debugging-Meldungen für VS1011-Ansteuerung
//#define DEBUG_PLAYER			// Debugging-Meldungen für Player
//#define DEBUG_INPUT			// Debugging-Meldungen für Tasteneingabe
//#define DEBUG_FILE_BROWSER	// Debugging-Meldungen für Datei-Browser

//=======================================


#define PRINT_LINE_NR()	global_printLineNr(PSTR(__FILE__), __LINE__);


/* -------------------------------- globale Variablen ------------------------------------- */

struct fat_fs_struct *global_pFatFileSystem;
struct partition_struct* global_pPartition;


char global_filename[FILENAME_MAXLEN];


// Variablen mit Einstellungen.
uint8_t settings_backlight; 		// Helligkeit der Display-Hintergrundbeleuchtung
uint8_t settings_backlight_time; 	// Dauer in Sekunden, für die die Hintergrundbeleuchtung
									// aktiviert bleibt, nachdem keine Tasteneingabe mehr erfolgt
									// ist
uint8_t settings_volume; 			// Wiedergabelautstärke
uint8_t settings_bassAmplitude; 	// Amplitude des Bass Enhancers
uint8_t settings_bassFrequency; 	// Grenzfrequenz des Bass Enhancers
uint8_t settings_trebleAmplitude;	// Amplitude des Treble Enhancers
uint8_t settings_trebleFrequency; 	// Grenzfrequenz des Treble Enhancers
uint8_t settings_theme; 			// Farbschema für die GUI


// Menüzustände

int8_t global_menustate;			// Der aktuelle Zustand des Menüs

// Konstanten für Menüzustände
enum
{
	STATE_BOOTING=0,
	STATE_PLAYER,
	STATE_MAINMENU,
	STATE_BROWSER_MAINMENU, //Wenn der Dateibrowser vom Hauptmenü aus aufgerufen wurde
	STATE_BROWSER_PLAYER, //Wenn der Dateibrowser vom Playerbildschirm aus aufgerufen wurde
	STATE_LIGHTING,
	STATE_THEMES,
	STATE_EQUALIZER,
	STATE_SHUTDOWN,
};


/* ------------------------------------ Prototypen -------------------------------------------
 *
 * Verschiedene Funktionen, die global benötigt werden.
 *
 */

#if testkk
bool global_fileFilterCallback(const struct fat_dir_entry_struct *pFatEntry);
#endif

void global_idleThread();
void global_timer_init();
void global_timer_set(uint8_t seconds);
void global_timer_reset();
void global_timer_stop();
void global_shutdownSystem();
#if testkk
char* getTrimmedText(const char *text, uint8_t maxWidth, lcd_pgmPointer_t pFont);
#endif
char* getCopiedString(const char *text);
#if testkk
char* getCopiedString_P(PGM_P text_P);
#endif

#ifdef DEBUG
#if testkk
void global_printLineNr(PGM_P filename, uint16_t line);
#endif
#endif

//=======================================

/* ------------------------ allgemeine Einstellungen ------------------------ */


// Gibt die Genauigkeit des Timers an: 1000 / GLOBAL_TIMER_DIVISOR = ms
#define GLOBAL_TIMER_DIVISOR 				40 //25 ms

// Maximal unterstützte Dateinamenlänge für einen FAT Eintrag
#define FILENAME_MAXLEN						128

// Größe eines Puffer-Frames, das der Player von SD-Karte läd und an den den VS1011 sendet
#define PLAYER_SEND_BUFFER_SIZE				64

// Größe des Puffers, der zum lesen von ID3 Tags verwendet wird
// Muss mindestens 30 Byte groß sein! Maximale ID3v2 Feld-Lesegröße wird zusätzlich
// in id3.h definiert (ID3_MAXIMAL_ID3V2_FIELD_SIZE)
#define ID3_BUFFER_SIZE						64

// Text, der während des ausschaltens angezeigt wird
#define SHUTDOWN_TEXT						"Goodbye"

// Die folgenden Konstanen geben an, ob der Player die Wiedergabe forsetzt, falls
// der File Browser bzw. das Hauptmenü angezeigt werden. Ansonsten wird die
// Wiedergabe auf Pause gesetzt
#define PLAYER_PAUSE_IN_FILE_BROWSER		true
#define PLAYER_PAUSE_IN_MAIN_MENU			false


/* ------------------- Einstellungen für GUI der Bootsequenz ---------------- */

// Gibt an, ob die Boot-Animation angezeigt werden soll
#define BOOTSEQUENCE_SHOW_ANIMATION			true

// Font für Text in Bootsequenz
#define BOOTSEQUENCE_FONT					gui_elements_pFont

// Y-Koordinate für Meldungstext
#if BOOTSEQUENCE_SHOW_ANIMATION
	#define BOOTSEQUENCE_MESSAGE_TEXT_Y		120
#else
	#define BOOTSEQUENCE_MESSAGE_TEXT_Y		(LCD_BOTTOM / 2 - lcd_getFontHeight(BOOTSEQUENCE_FONT))
#endif

// Meldungstext, der angezeigt wird, wenn keine SD-Karte erkannt wurde
#define BOOTSEQUENCE_NO_SD_CARD_TEXT		"Bitte SD-Karte einlegen"

// Y-Koordinate für Versionsinformations-Text
#define BOOTSEQUENCE_VERSION_TEXT_Y			(LCD_BOTTOM - 20)

#define BOOTSEQUENCE_LOGO_Y					50


/* --------------------------- Werteeinstellungen ------------------------------
 *
 * Gibt an, welche Wertebereiche für verschiedene Einstellungen gültig sind.
 *
 */
#define LCD_BACKLIGHT_MIN 0
#define LCD_BACKLIGHT_DEFAULT 100
#define LCD_BACKLIGHT_MAX 200

#define LCD_BACKLIGHT_TIME_MIN 0
#define LCD_BACKLIGHT_TIME_DEFAULT 10
#define LCD_BACKLIGHT_TIME_MAX 31

#define PLAYER_VOLUME_MIN 0
#define PLAYER_VOLUME_DEFAULT 20
#define PLAYER_VOLUME_MAX 40

#define PLAYER_BASS_AMPLITUDE_MIN 0
#define PLAYER_BASS_AMPLITUDE_DEFAULT 0
#define PLAYER_BASS_AMPLITUDE_MAX 15

#define PLAYER_BASS_FREQUENCY_MIN 0
#define PLAYER_BASS_FREQUENCY_DEFAULT 60
#define PLAYER_BASS_FREQUENCY_MAX 15

// Von den Werten wird noch jeweils 8 subtrahiert!
#define PLAYER_TREBLE_AMPLITUDE_MIN 0
#define PLAYER_TREBLE_AMPLITUDE_DEFAULT 8
#define PLAYER_TREBLE_AMPLITUDE_MAX 15

#define PLAYER_TREBLE_FREQUENCY_MIN 0
#define PLAYER_TREBLE_FREQUENCY_DEFAULT 10
#define PLAYER_TREBLE_FREQUENCY_MAX 15

#define GUI_THEME_MIN 0
#define GUI_THEME_DEFAULT 0
#define GUI_THEME_MAX 2

// Anzahl der Sekunden, nach denen das Player-Menü wieder auf das Wiedergabekontroll
// Menü gesetzt wird
#define PLAYER_MENU_RESET_TIMER 8


// Zeichenkette, die benutzt wird um Fehler in der GUI zu signalisieren (bspw.
// Fehler beim Laden von Menüeinträgen)
#define TEXT_ERROR		"FEHLER"


//===================================================
#include <stdint.h>
#include <stdbool.h>


// ------------------------ Präprozessor-Konstanten ----------------------------

// VS1011 SCI Register
#define VS1011_SCI_MODE			0x0
#define VS1011_SCI_STATUS		0x1
#define VS1011_SCI_BASS			0x2
#define VS1011_SCI_CLOCKF		0x3
#define VS1011_SCI_DECODE_TIME	0x4
#define VS1011_SCI_AUDATA		0x5
#define VS1011_SCI_WRAM			0x6
#define VS1011_SCI_WRAMADDR		0x7
#define VS1011_SCI_HDAT0		0x8
#define VS1011_SCI_HDAT1		0x9
#define VS1011_SCI_AIADDR		0xA
#define VS1011_SCI_VOL			0xB
#define VS1011_SCI_AICTRL0		0xC
#define VS1011_SCI_AICTRL1		0xD
#define VS1011_SCI_AICTRL2		0xE
#define VS1011_SCI_AICTRL3		0xF

// VS1011 modes
#define VS1011_SM_DIFF			0
#define VS1011_SM_LAYER12		1
#define VS1011_SM_RESET			2
#define VS1011_SM_OUTOFWAV		3
#define VS1011_SM_SETTOZERO1	4
#define VS1011_SM_TESTS			5
#define VS1011_SM_STREAM		6
#define VS1011_SM_SETTOZERO2	7
#define VS1011_SM_DACT			8
#define VS1011_SM_SDIORD		9
#define VS1011_SM_SDISHARE		10
#define VS1011_SM_SDINEW		11
#define VS1011_SM_SETTOZERO3	12
#define VS1011_SM_SETTOZERO4	13

// Maximale Abschwächung (in dB) für vs1011_setVolume()
#define VS1011_MAXIMUM_ATTENUATION	127


// ------------------------------ Prototypen -----------------------------------


// ---------------------------- Initialisierung --------------------------------
// Konfiguriert die I/O-Pins des Controllers für den VS1011
void vs1011_configurePins();

// Führt einen Reset des VS1011 durch und initialisiert ihn. Vorher sollte
// vs1011_configurePins() aufgerufen werden.
void vs1011_init();


// ----------------------------- Datentransfer ---------------------------------

// Ließt ein Befehlsregister des VS1011 aus
uint16_t vs1011_sciRead(uint8_t address, bool bWaitForDreq);

// Schreibt ein Befehlsregister des VS1011
void vs1011_sciWrite(uint8_t address, uint16_t data, bool bWaitForDreq);

// Sendet ein Datenbyte and das Serial Data Interface des VS1011
void vs1011_sdiWriteByte(uint8_t byte, bool bWaitForDreq);

// Sendet ein Puffer mit Daten and das Serial Data Interface des VS1011
void vs1011_sdiWriteBuffer(uint8_t *pBuffer, uint16_t offset, uint16_t length);

// Leert den Wiedergabepuffer des VS1011 durch senden einer Folger von Null-Bytes
void vs1011_flushBuffer();

// Wartet so lange blockierend, bis der VS1011 durch den DREQ Pin Empfangsbereitschaft
// signalisiert
void vs1011_waitForDreq();

// Gibt zurück, ob der DREQ Pin des VS1011 High oder Low ist
bool vs1011_getDreq();


// ---------------------------------- SPI --------------------------------------

// Wählt explizit das Serial Command Interface des VS1011 am SPI-Bus aus.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_selectSci();

// Wählt explizit das Serial Data Interface des VS1011 am SPI-Bus aus.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_selectSdi();

// Wählt den VS1011 explizit vom SPI-Bus ab.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_unselectSpi();


// --------------------------- Wiedergabesteuerung -----------------------------

// Setzt die Wiedergabelautstärke der Endstufe des VS1011 (angegeben in Dämpfung
// für linken und rechten Kanal in dB).
void vs1011_setVolume(uint8_t attenuationLeftDb, uint8_t attenuationRightDb);

// Setzt den Bass / Treble-Enhancer des VS1011. Amplitudenwerte von 0 deaktivieren
// den jeweiligen Filter.
void vs1011_setEqualizer(float fBassAmplitude, float fBassFrequency,
						 float fTrebleAmplitude, float fTrebleFrequency);

// Liefert die Wiedergabedauer des aktuellen Datenstroms des VS1011 in Sekunden.
// Der VS1011 addiert intern die Dauer jedes MP3-Frames, daher muss ggf. beim
// senden eines neuen Datenstroms vs1011_resetPlaybackDuration() aufgerufen werden,
// um den Zähler zurück zu setzen.
uint16_t vs1011_getPlaybackDuration();

// Setzt den Wiedergabezähler des VS1011 auf Null zurück.
void vs1011_resetPlaybackDuration();

// Liefert die Abtast- und Bitrate des zuletzt dekodierten MP3-Frames des aktuellen
// Datenstroms. Nicht-benötigte Zeiger-Parameter können auf NULL gesetzt werden.
bool vs1011_getStreamData(uint16_t *pSampleRate, uint16_t *pBitRate);

//==========================================


// ---------------------------- Initialisierung --------------------------------
// Konfiguriert die I/O-Pins des Controllers für den VS1011
void vs1011_configurePins();

// Führt einen Reset des VS1011 durch und initialisiert ihn. Vorher sollte
// vs1011_configurePins() aufgerufen werden.
void vs1011_init();


// ----------------------------- Datentransfer ---------------------------------

// Ließt ein Befehlsregister des VS1011 aus
uint16_t vs1011_sciRead(uint8_t address, bool bWaitForDreq);

// Schreibt ein Befehlsregister des VS1011
void vs1011_sciWrite(uint8_t address, uint16_t data, bool bWaitForDreq);

// Sendet ein Datenbyte and das Serial Data Interface des VS1011
void vs1011_sdiWriteByte(uint8_t byte, bool bWaitForDreq);

// Sendet ein Puffer mit Daten and das Serial Data Interface des VS1011
void vs1011_sdiWriteBuffer(uint8_t *pBuffer, uint16_t offset, uint16_t length);

// Leert den Wiedergabepuffer des VS1011 durch senden einer Folger von Null-Bytes
void vs1011_flushBuffer();

// Wartet so lange blockierend, bis der VS1011 durch den DREQ Pin Empfangsbereitschaft
// signalisiert
void vs1011_waitForDreq();

// Gibt zurück, ob der DREQ Pin des VS1011 High oder Low ist
bool vs1011_getDreq();


// ---------------------------------- SPI --------------------------------------

// Wählt explizit das Serial Command Interface des VS1011 am SPI-Bus aus.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_selectSci();

// Wählt explizit das Serial Data Interface des VS1011 am SPI-Bus aus.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_selectSdi();

// Wählt den VS1011 explizit vom SPI-Bus ab.
// Muss normalerweise zum Senden und Empfangen nicht aufgerufen werden (wird
// entsprechend implizit aufgerufen).
void vs1011_unselectSpi();


// --------------------------- Wiedergabesteuerung -----------------------------

// Setzt die Wiedergabelautstärke der Endstufe des VS1011 (angegeben in Dämpfung
// für linken und rechten Kanal in dB).
void vs1011_setVolume(uint8_t attenuationLeftDb, uint8_t attenuationRightDb);

// Setzt den Bass / Treble-Enhancer des VS1011. Amplitudenwerte von 0 deaktivieren
// den jeweiligen Filter.
void vs1011_setEqualizer(float fBassAmplitude, float fBassFrequency,
						 float fTrebleAmplitude, float fTrebleFrequency);

// Liefert die Wiedergabedauer des aktuellen Datenstroms des VS1011 in Sekunden.
// Der VS1011 addiert intern die Dauer jedes MP3-Frames, daher muss ggf. beim
// senden eines neuen Datenstroms vs1011_resetPlaybackDuration() aufgerufen werden,
// um den Zähler zurück zu setzen.
uint16_t vs1011_getPlaybackDuration();

// Setzt den Wiedergabezähler des VS1011 auf Null zurück.
void vs1011_resetPlaybackDuration();

// Liefert die Abtast- und Bitrate des zuletzt dekodierten MP3-Frames des aktuellen
// Datenstroms. Nicht-benötigte Zeiger-Parameter können auf NULL gesetzt werden.
bool vs1011_getStreamData(uint16_t *pSampleRate, uint16_t *pBitRate);

//==================================================================



// --------------------------- interne Funktionen ------------------------------

// ______________________
//
//  vs1011_spiTransmit()
// ______________________
//
// 		Interne Funktion für SPI-Übertragung (Senden und Empfangen).
//
//	Parameter:
//
//		uint8_t data			Zu sendendes Datenbyte
//
//		bool bWaitForDreq		Wenn bWaitForDreq true ist wird vor der Übertragung
//								auf High-Pegel am DREQ-Pin gewartet
//
//	Rückgabewert:
//
//		uint8_t					Empfangenes Datenbyte
//
uint8_t vs1011_spiTransmit(uint8_t data, bool bWaitForDreq)
{
	if (bWaitForDreq)
		vs1011_waitForDreq();
#if testkk
	SPDR = data;
  	while (!(SPSR & (1 << SPIF)));
	return SPDR;
#endif
}



// _____________________________
//
//  vs1011_restrictFloatValue()
// _____________________________
//
//		Interne Funktion, die einen float-Wert auf den Wertebereich [0.0, 1.0]
//		beschränkt
//
//	Parameter:
//
//		float *f		Zeiger auf den zu beschränkenden float-Wert
//
void vs1011_restrictFloatValue(float *f)
{
	if (*f < 0)
		*f = 0;
	else if (*f > 1)
		*f = 1;
}



// ------------------------- öffentliche Funktionen ----------------------------


// ________________________
//
//  vs1011_configurePins()
// ________________________
//
// 		Konfiguriert die Controller I/O Pins für den VS1011 (Richtungsregister
//		und Initialisierung).
//
void vs1011_configurePins()
{
	// I/O Pins Konfigurieren
	VS1011_CONFIGURE_PIN_DREQ();
	VS1011_CONFIGURE_PIN_XRESET();
	VS1011_CONFIGURE_PIN_XDCS();
	VS1011_CONFIGURE_PIN_XCS();
	vs1011_unselectSpi();
}



// _______________
//
//  vs1011_init()
// _______________
//
//		Initialisiert den VS1011.
//
//		Beendet dabei den Reset und setzt Übertragungsmodus und Takt.
//		Vorher sollte vs1011_configurePins() aufgerufen werden, um die I/O
//		Pins am Controller zu konfigurieren.
//
void vs1011_init()
{
	// Reset
	VS1011_SET_XRESET();
	_delay_ms(15);

	// SDI New Mode setzen (SM_SDINEW im Mode-Register)
	vs1011_sciWrite(VS1011_SCI_MODE, (1 << VS1011_SM_SDINEW), true);

	// VS1011 Takt setzen, 12.228 Mhz mit Takverdoppler
	vs1011_sciWrite(VS1011_SCI_CLOCKF, 0x9800, true);

	// Samplerate auf 44100 Hz setzen (siehe VS1011 application note)
	//vs1011_sciWrite(VS1011_SCI_AUDATA, 0xAC45, true);
}
void init_vs10xx(){
	//pins
	init_ssp0_1();

	pinDIRout(0,22);//xccs
	pinSET(0,22);
	pinDIRin(0,19);
	pinDIRout(0,20);//xdcs
	pinSET(0,20);

	_delay_ms(15);


	// SDI New Mode setzen (SM_SDINEW im Mode-Register)
	vs1011_sciWrite(VS1011_SCI_MODE, (1 << VS1011_SM_SDINEW), true);

	// VS1011 Takt setzen, 12.228 Mhz mit Takverdoppler
	vs1011_sciWrite(VS1011_SCI_CLOCKF, 0x9800, true);

	// Samplerate auf 44100 Hz setzen (siehe VS1011 application note)
	vs1011_sciWrite(VS1011_SCI_AUDATA, 0xAC45, true);

}

// ____________________
//
//  vs1011_selectSci()
// ____________________
//
// 		Wählt das Serial Command Interface (SCI) des VS1011 am SPI-Bus an.
//
void vs1011_selectSci()
{
	VS1011_SET_XDCS();
	VS1011_CLEAR_XCS();
}


// ____________________
//
//  vs1011_selectSdi()
// ____________________
//
// 		Wählt das Serial Data Interface (SDI) des VS1011 am SPI-Bus an.
//
void vs1011_selectSdi()
{
	VS1011_SET_XCS();
	VS1011_CLEAR_XDCS();
}



// ______________________
//
//  vs1011_unselectSpi()
// ______________________
//
// 		Wählt den VS1011 komplett vom SPI Bus ab (SDI und SCI).
//
void vs1011_unselectSpi()
{
	VS1011_SET_XCS();
	VS1011_SET_XDCS();
}




// __________________
//
//  vs1011_sciRead()
// __________________
//
// 		Liest ein SCI (Serial Control Interface) Register des VS1011.
//
//	Parameter:
//
//		uint8_t address			Adresse des Registers
//
//		uint16_t dw				Zu schreibender Wert
//
//		bool bWaitForDreq		Wenn bWaitForDreq true ist wird vor dem Lesen
//								auf High-Pegel am DREQ-Pin gewartet
//
//	Rückgabewert:
//
//		uint16_t				Registerwert
//
uint16_t vs1011_sciRead(uint8_t address, bool bWaitForDreq)
{
	uint8_t dwHigh, dwLow;

	VS1011_SET_SPI_CLOCK_READ();
	VS1011_CLEAR_XCS();

	vs1011_spiTransmit(0x03, bWaitForDreq);		// Lesemodus
	vs1011_spiTransmit(address, false);
	dwHigh = vs1011_spiTransmit(0, false);		// Datenwort: don't care
	dwLow = vs1011_spiTransmit(0, false);

	VS1011_SET_XCS();

	return (dwHigh << 8) | dwLow;
}



// ___________________
//
//  vs1011_sciWrite()
// ___________________
//
// 		Sendet einen Befehl an das  Serial Control Interface (SCI) des VS1011
//		(schreibt ein Befehlsregister).
//
//	Parameter:
//
//		uint8_t address			Adresse des Registers
//
//		uint16_t data			Zu schreibender 16-Bit Registerwert
//
//		bool bWaitForDreq		Wenn bWaitForDreq true ist wird vor dem Senden
//								auf High-Pegel am DREQ-Pin gewartet
//
void vs1011_sciWrite(uint8_t address, uint16_t data, bool bWaitForDreq)
{
	VS1011_SET_SPI_CLOCK_WRITE();

	VS1011_CLEAR_XCS();

	vs1011_spiTransmit(0x02, bWaitForDreq);		// Schreibmodus
	vs1011_spiTransmit(address, false);			// Register Adresse
	vs1011_spiTransmit(data >> 8, false);		// Höherweriges Byte
	vs1011_spiTransmit(data & 0xFF, false);		// Niederwertiges Byte

	VS1011_SET_XCS();
}


// _______________________
//
//  vs1011_sdiWriteByte()
// _______________________
//
// 		Sendet ein Byte an das Serial Data Interface (SDI) des VS1011.
//
//	Parameter:
//
//		uint8_t byte			Zu sendendes Byte
//
//		bool bWaitForDreq		Wenn bWaitForDreq true ist wird vor dem Senden
//								auf High-Pegel am DREQ-Pin gewartet
//
void vs1011_sdiWriteByte(uint8_t byte, bool bWaitForDreq)
{
	VS1011_SET_SPI_CLOCK_WRITE();
    VS1011_CLEAR_XDCS();

	vs1011_spiTransmit(byte, bWaitForDreq);

    VS1011_SET_XDCS();
}



// _________________________
//
//  vs1011_sdiWriteBuffer()
// _________________________
//
// 		Sendet Daten in einem Puffer-Array an das Serial Data Interface (SDI) des
//		VS1011. Wartet vor dem Senden jedes Bytes auf High-Pegel am DREQ-Pin.
//
//		Wenn pBuffer = NULL ist werden als Daten Null-Bytes gesendet.
//
//	Parameter:
//
//		uint8_t *pBuffer	Puffer-Array mit den zu sendenden Daten oder NULL,
//							um eine Folge von Null-Bytes zu senden
//
//		uint16_t offset		Versatz ab dem auf die Pufferdaten zugegriffen wird
//
//		uint16_t length 	Anzahl der aus dem Puffer zu sendenden Bytes.
//							Es muss gelten: (offset + length) <= sizeof (buffer)
//
void vs1011_sdiWriteBuffer(uint8_t *pBuffer, uint16_t offset, uint16_t length)
{
	length += offset;

	VS1011_SET_SPI_CLOCK_WRITE();
    VS1011_CLEAR_XDCS();
uint16_t i;
	if (pBuffer == NULL) {
		// Folge von Nullen senden
		for ( i= offset; i < length; i++)
			vs1011_spiTransmit(0, true);
	} else {
		// Puffer senden
		for ( i = offset; i < length; i++)
			vs1011_spiTransmit(pBuffer[i], true);
	}

    VS1011_SET_XDCS();
}



// ______________________
//
//  vs1011_waitForDreq()
// ______________________
//
//	Wartet solange blockierend, bis der DREQ Pin des VS1011 Empfangsbereitschaft
//	signalisiert.
//
void vs1011_waitForDreq()
{
//	while (!VS1011_GET_DREQ());
while(!(LPC_GPIO0->FIOPIN)&&_BV(19));
}

// ______________________
//
//  vs1011_getDreq()
// ______________________
//
//   Gibt zurück, ob der DREQ Pin des VS1011 High oder Low ist
//
bool vs1011_getDreq()
{
	return VS1011_GET_DREQ();
}


// ______________________
//
//  vs1011_flushBuffer()
// ______________________
//
//	Leert den Abspielpuffer des VS1011, indem 2048 Null-Bytes an das Dateninterface
//	gesendet werden. Dient dazu Störgeräusche nach dem Beenden des Abspielens
//	zu vermeiden.
//
void vs1011_flushBuffer()
{
#if testkk
	vs1011_sdiWriteBuffer(NULL, 0, VS1011_FLUSH_BUFFER_BYTES);
#endif
}



// ____________________
//
//  vs1011_setVolume()
// ____________________
//
// 		Setzt die Wiedergabe-Lautstärke des VS1011.
//
//	Parameter:
//
//		uint8_t attenuationLeftDb	Abschwächung des linken Kanals in dB
//
//		uint8_t attenuationRightDb	Abschwächung des rechten Kanals in dB
//
void vs1011_setVolume(uint8_t attenuationLeftDb, uint8_t attenuationRightDb)
{
	vs1011_sciWrite(VS1011_SCI_VOL, attenuationLeftDb * 512 + attenuationRightDb * 2, true);
}



// _______________________
//
//  vs1011_setEqualizer()
// _______________________
//
//		Setzt die Bass und Treble Enhancer Einstellungen des VS1011. Parameter
//		Werden im Bereich von 0.0 - 1.0 angegeben, wobei Amplituden von 0.0
//		den jeweiligen Filter deaktiveren.
//		Die Parameter werden ggf. auf den Wertebereich eingeschränkt.
//
//	Parameter:
//
//		float fBassAmplitude		Wertebereich: [0.0, 1.0]
//
//		float fBassFrequency		Wertebereich: [0.0, 1.0]
//
//		float fTrebleAmplitude		Wertebereich: [0.0, 1.0]
//
//		float fTrebleFrequency		Wertebereich: [0.0, 1.0]
//
void vs1011_setEqualizer(float fBassAmplitude, float fBassFrequency,
						 float fTrebleAmplitude, float fTrebleFrequency)
{
	// Bass Frequenz: 4 bit, 10 Hz Schritte
	// Bass Amplitude: 4 bit, 1 dB Schritte, 0 = aus
	// Treble Frequenz: 4 bit, 1000 Hz Schritte
	// Treble Amplitude: 4 bit, 1.5 dB Schritte, 0 = aus

	vs1011_restrictFloatValue(&fBassAmplitude);
	vs1011_restrictFloatValue(&fBassFrequency);
	vs1011_restrictFloatValue(&fTrebleAmplitude);
	vs1011_restrictFloatValue(&fTrebleFrequency);

	vs1011_sciWrite(VS1011_SCI_BASS,
					((uint16_t) (fTrebleAmplitude * 15) << 12) |
					((uint16_t) (fTrebleFrequency * 15) <<  8) |
					((uint16_t) (fBassAmplitude * 15)   <<  4) |
					((uint16_t) (fBassFrequency * 15)),
				    true);
}



// ______________________________
//
//  vs1011_getPlaybackDuration()
// ______________________________
//
//		Liefert die Wiedergabe dauer des aktuell an den VS1011 gesendeten
//		Datenstroms in Sekunden zurück.
//
uint16_t vs1011_getPlaybackDuration()
{
	return vs1011_sciRead(VS1011_SCI_DECODE_TIME, true);
}



// ______________________________
//
//  vs1011_resetPlaybackDuration()
// ______________________________
//
//		Setzt die Wiedergabedauer des VS1011 auf 0 zurück (der VS1011 Decoder
//		zählt nur die Sekunden hoch, die er bereits dekodiert hat sei dem letzen
//		Reset).
//
void vs1011_resetPlaybackDuration()
{
	vs1011_sciWrite(VS1011_SCI_DECODE_TIME, 0, true);
}



// ________________________
//
//  vs1011_getStreamData()
// ________________________
//
//		Liefert die Abstastrate in Hertz und Bitrate in kbit/s des zuletzt
//		dekodierten MP3-Frames des Datenstroms, falls es sich dabei um einen
//		MPEG 1.0 Layer III handelte. Für andere Frames werden keine gültigen
//		Werte zurückgeliefert und die Funktion gibt false zurück, um die
//		Implementation kompakt (Programmspeicher).
//
//	Parameter:
//
//		uint16_t *pSampleRate		Zeiger auf eine uint16_t Variable, in die die
//									Abtastrate geschrieben wird, oder NULL, falls
//									Abtastrate nicht benötigt wird
//
//		uint16_t *pBitRate			Zeiger auf eine uint16_t Variable, in die die
//									Bitrate geschrieben wird, oder NULL, falls
//									falls Bitrate nicht benötigt wird
//
bool vs1011_getStreamData(uint16_t *pSampleRate, uint16_t *pBitRate)
{
	uint16_t headerData0 = vs1011_sciRead(VS1011_SCI_HDAT0, true);
	uint16_t headerData1 = vs1011_sciRead(VS1011_SCI_HDAT1, true);
	uint8_t bitRateBits, sampleRateBits, mpegId, layer;

	// Daten aus den Header-Registern extrahieren
	layer = 4 - ((headerData1 >> 1) & 3);
	if (layer > 3)
		return false;

	mpegId = (headerData1 >> 3) & 3;
	if ((mpegId != 3) || (layer != 3)) {
		*pSampleRate = 0;
		*pBitRate = 0;
		return false;
	}


	// --------------------------- Abtastrate ----------------------------------

	if (pSampleRate != NULL) {
		sampleRateBits = (headerData0 >> 10) & 3;

		switch (sampleRateBits) {
			case 0: *pSampleRate = 44100; break;
			case 1: *pSampleRate = 48000; break;
			case 2: *pSampleRate =  32000; break;
			default: *pSampleRate = 0; break;
		}
	}


	// ---------------------------- Bitrate -----------------------------------

	if (pBitRate != NULL) {
		bitRateBits = (headerData0 >> 12) & 0xF;

		if ((bitRateBits == 0) || (bitRateBits == 15)) {
			// ungültig
			*pBitRate = 0;
			return false;
		} else if (bitRateBits < 6) {
			// 32 - 64 kbps
			*pBitRate = 24 + bitRateBits * 8;
		} else if (bitRateBits < 10) {
			// 80 - 128 kbps
			*pBitRate = 80 + (bitRateBits - 6) * 16;
		} else if (bitRateBits < 14) {
			// 160 - 256 kbps
			*pBitRate = 160 + (bitRateBits - 10) * 32;
		} else {
			*pBitRate = 320;
		}
	}

	return true;
}
//===================================
//===================================
//===================================


// -------------------------- globale Variablen --------------------------------

struct fat_file_struct *player_pFatFile;
uint8_t player_isPlaying;
uint32_t player_fileSize;
uint32_t player_dataOffset;

// File Browser und Tag,
// Global, damit aus player_gui.c direkt darauf zugegriffen werden kann (ohne getter)
#if testkk
id3_tag_t player_tag;
fatFileBrowser_t player_fileBrowser;
#endif


// ------------------------------ Prototypen -----------------------------------

void player_configurePins();
#if testkk
uint8_t player_init(struct fat_fs_struct *pFatFileSystem, fatFileBrowser_boolCallback_t fileFilterCallback);
#endif

uint8_t player_thread();
void player_pause();
void player_resume();
uint8_t player_playPrevious();
uint8_t player_playNext();
uint8_t player_playCurrent();

uint16_t player_getPlackbackDuration();
bool player_getStreamData(uint16_t *pSampleRate, uint16_t *pBitRate);

uint8_t player_volumeUp();
uint8_t player_volumeDown();
void player_setVolume(uint8_t volume);
void player_setBass(uint8_t bass);
void player_setHeights(uint8_t bass);

void player_setBassAmplitude(uint8_t amplitude);
void player_setBassFrequency(uint8_t frequency);
void player_setTrebleAmplitude(uint8_t amplitude);
void player_setTrebleFrequency(uint8_t frequency);

//===================================

#ifdef DEBUG_PLAYER
#include <stdio.h>
#include <avr/pgmspace.h>
#endif



// ---------------------------- globale Variablen ------------------------------

// Sendepuffer für Player, um MP3-Daten an den VS1011-Decoder zu senden
uint8_t player_sendBuffer[PLAYER_SEND_BUFFER_SIZE];

// FAT-Dateihandle des Players
struct fat_file_struct *player_pFatFile = NULL;

// Größe der aktuell geladenen FAT-Datei
uint32_t player_fileSize = 0;

// Offset in der FAT-Datei, an dem die MP3-Daten anfangen
uint32_t player_dataOffset;


// ---------------------------- Player Funktionen ------------------------------

void player_configurePins()
{
	vs1011_configurePins();
}

#if testkk
uint8_t player_init(struct fat_fs_struct *pFatFileSystem, fatFileBrowser_boolCallback_t fileFilterCallback)
#endif
uint8_t player_init(struct fat_fs_struct *pFatFileSystem, uint8_t fileFilterCallback)
{
	bool b;

	player_pFatFile = NULL;
	player_isPlaying = false;
#if testkk
id3_initializeTagFields(&player_tag);
#endif

	vs1011_init();
#if testkk
	b = fatFileBrowser_init(&player_fileBrowser, pFatFileSystem, fileFilterCallback, NULL);
#endif

	#ifdef DEBUG_PLAYER
		if (!b)
			printf_P(PSTR("FEHLER: fatFileBrowser_init() fehlgeschlagen (%s, Zeile %d)\n"), __FILE__, __LINE__ - 2);
	#endif

	return b;
}


void player_closeFile()
{
	cli();

	fat_close_file(player_pFatFile);
	player_pFatFile = NULL;

	sei();
}


uint8_t player_playFile(int8_t direction)
{
	uint8_t buffer[ID3_BUFFER_SIZE];	// Puffer zum Lesen des ID3-Tags

	// Interrupts deaktivieren
	cli();

	// Alte Datei schließen (falls geöffnet)
	fat_close_file(player_pFatFile);

	// Alte Member-Strings des Tags freigeben
#if testkk
	id3_freeTagFields(&player_tag);
#endif

	// Lautstärke des VS1011 auf stumm stellen
	vs1011_setVolume(VS1011_MAXIMUM_ATTENUATION, VS1011_MAXIMUM_ATTENUATION);

	// Den Puffer des VS1011 leeren
	vs1011_flushBuffer();

	// Wenn direction = 0, dann nicht den nächsten Eintrag holen.
	if (direction != 0) {
#if testkk

		if (!fatFileBrowser_loadEntry(&player_fileBrowser, direction, FAT_FILE_BROWSER_ENTRY_FILE)) {
#endif
			sei();
			return 0;
		}
	}

	// Datei öffnen, die im File Browser selektiert ist
#if testkk
	player_pFatFile = fat_open_file(player_fileBrowser.pFatFileSystem, player_fileBrowser.pFatEntry);
	if (player_pFatFile == NULL) {
		sei();
		return 0;
	}
#endif


	// Dateigröße ermitteln
#if testkk
	player_fileSize = fat_get_file_size(player_pFatFile);
	if (player_fileSize == 0) {
		// Leere Datei
		fat_close_file(player_pFatFile);
		sei();
		return 0;
	}
#endif
#if testkk
	// ID3-Tags aus Datei auslesen, setzt Dateiposition vor Beginn der MP3-Daten
	id3_readTagFromFile(&player_tag, player_pFatFile, buffer, sizeof (buffer));
	player_dataOffset = fat_get_file_position(player_pFatFile);

	// Widergabedauer des VS1011 zurücksetzen
	vs1011_resetPlaybackDuration();

	// Player-Status auf "abspielend"
	player_isPlaying = 1;

	// Alte Lautstärke wiederherstellen
	player_setVolume(settings_volume);

	// Globalen Dateinamen übernehmen
	strcpy(global_filename, player_fileBrowser.pFatEntry->long_name);
#endif


	// Player GUI aktualisieren lassen
	player_gui_playbackSeconds = 0;
	player_gui_streamDataTimerCounter = 0;

	// Interrupts wieder aktvieren
	sei();

	player_gui_refresh();

#if testkk
	return 1;
}
#endif

#if testkk
uint8_t player_playPrevious()
{
	if (player_fileBrowser.pFatEntry == NULL)
		// Keine Datei im Filebrowser geladen, Fehler
		return 0;
	else if (player_fileBrowser.index[FAT_FILE_BROWSER_ENTRY_FILE] == 0)
		// Überlauf zur letzten Datei
		return player_playFile(FAT_FILE_BROWSER_LAST);
	else
		return player_playFile(FAT_FILE_BROWSER_PREVIOUS);
}



uint8_t player_playNext()
{
	if (player_fileBrowser.pFatEntry == NULL)
		// Keine Datei im Filebrowser geladen, erste Datei der Liste laden
		return player_playFile(FAT_FILE_BROWSER_FIRST);
	else if (player_fileBrowser.index[FAT_FILE_BROWSER_ENTRY_FILE] == player_fileBrowser.nEntries[FAT_FILE_BROWSER_ENTRY_FILE] - 1)
		// Überlauf zur ersten Datei
		return player_playFile(FAT_FILE_BROWSER_FIRST);
	else
		return player_playFile(FAT_FILE_BROWSER_NEXT);
}
#endif


uint8_t player_playCurrent()
{
	return player_playFile(0);
}

void player_pause()
{
	player_isPlaying = 0;
}


void player_resume()
{
	player_isPlaying = 1;
}


uint16_t player_getPlackbackDuration()
{
	return vs1011_getPlaybackDuration();
}


bool player_getStreamData(uint16_t *pSampleRate, uint16_t *pBitRate)
{
	return vs1011_getStreamData(pSampleRate, pBitRate);
}


// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= Lautstärke =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=

void player_setVolume(uint8_t volume)
{
	settings_volume = volume;

	// Lautstärke in Dämpfung umrechnen
	vs1011_setVolume(PLAYER_VOLUME_MAX - volume, PLAYER_VOLUME_MAX - volume);
}


uint8_t player_volumeDown()
{
	if (settings_volume > PLAYER_VOLUME_MIN)
		player_setVolume(settings_volume - 1);

	return settings_volume;
}


uint8_t player_volumeUp()
{
	if (settings_volume < PLAYER_VOLUME_MAX)
		player_setVolume(settings_volume + 1);

	return settings_volume;
}




// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= Equalizer -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-=


void player_setEqualizerWithSavedSettings()
{
	vs1011_setEqualizer((float) settings_bassAmplitude / PLAYER_BASS_AMPLITUDE_MAX,
						(float) settings_bassFrequency / PLAYER_BASS_FREQUENCY_MAX,
						(float) settings_trebleAmplitude / PLAYER_TREBLE_AMPLITUDE_MAX,
						(float) settings_trebleFrequency / PLAYER_TREBLE_FREQUENCY_MAX);
}


void player_setBassAmplitude(uint8_t amplitude)
{
	settings_bassAmplitude = amplitude;
	player_setEqualizerWithSavedSettings();
}


void player_setBassFrequency(uint8_t frequency)
{
	settings_bassFrequency = frequency;
	player_setEqualizerWithSavedSettings();
}


void player_setTrebleAmplitude(uint8_t amplitude)
{
	settings_trebleAmplitude = amplitude;
	player_setEqualizerWithSavedSettings();
}


void player_setTrebleFrequency(uint8_t frequency)
{
	settings_trebleFrequency = frequency;
	player_setEqualizerWithSavedSettings();
}



// ----------------------------- player_thread() -------------------------------
//
//	Thread des Players. Sendet Daten aus der aktuellen Datei an den VS1011, falls
//	momentan eine Widergabe sattfindet.
//
//	Rückgabewert:
//		0: Nichts zu tun, Player hat keinen Spielauftrag
//		1: Daten wurden an den VS1011 gesendet
//		2: Das Dateiende wurde erreicht (oder die Datei konnte nicht gelesen werden)
//		   und die Datei wurde geschlossen
//
uint8_t player_thread()
{
	uint16_t bytesRead;

	if (player_isPlaying) {

		// vs1011_waitForDreq(); //FIXED: In diesem Thread sollte nicht blockiert werden.
		if (!vs1011_getDreq()) //Wenn der VS1011 keine Daten wünscht, Thread einfach verlassen statt zu warten.
			return 1;

		//Progressbar wird nur aktualisiert, wenn der VS1011 Daten gewünscht hat.
		if (global_menustate == STATE_PLAYER)
			// Player spielt und der Bildschirm wird angezeigt, also Progressbar anzeigen.
			player_gui_progressbar();

		// Interrupts deaktivieren
		cli();

		// Daten für VS1011 aus FAT-Datei lesen
		bytesRead = fat_read_file(player_pFatFile, player_sendBuffer, sizeof (player_sendBuffer));

		// Interrupts aktivieren

		if (bytesRead <= 0) {
			// Dateiposition am Ende angekommen
			player_closeFile();
			player_isPlaying = 0;

			sei();
			return 2;
		} else {
			// An VS1011 Senden
			vs1011_sdiWriteBuffer(player_sendBuffer, 0, bytesRead);

			sei();
			return 1;
		}
	} else
		return 0;
}
//===================================



#endif
